%{
#include <stdio.h>
#include <string.h>
#include "parser.tab.h"

#ifndef STANDALONE
extern void yyerror(const char *);
#else
void yyerror(const char *);
char *yyfpath;
#endif

int yylinenum = 1;
int numopt;
%}

%option noyywrap

%x SKETCH
%x OPTIONS
%x OMESG
%x ACTION

LABEL   [A-Za-z0-9]*
%%
<INITIAL>^#.*
<INITIAL>[ \t]*
<INITIAL>{LABEL}        return NAME;
<INITIAL>\n             yylinenum++;
<INITIAL>"{"            BEGIN(SKETCH); return LB;

<SKETCH>\n              yylinenum++; yymore();
<SKETCH>^--.*           {
                            int p=yyleng;
                            while (strncmp(&yytext[p-3],"\n--",3) != 0) {
                                p--;
                            }
                            yytext[p-2] = '\0';
                            numopt=0;
                            BEGIN(OPTIONS);
                            return TEXT;
                        }
<SKETCH>"}"             {
                            fprintf(stderr, "Lexer Error:%d:", yylinenum);
                            yyerror("sketch unterminated");
                            BEGIN(INITIAL);
                        }
<SKETCH>.*              yymore();

<OPTIONS>\n             yylinenum++;
<OPTIONS>[ \t]+
<OPTIONS>"}"              {
                            if (numopt < 1) {
                                fprintf(stderr, "Lexer Error:%d:", yylinenum);
                                yyerror("option has no action");
                                BEGIN(INITIAL);
                            } else {
                                BEGIN(INITIAL);
                                return RB;
                            }
                        }
<OMESG>\n               yylinenum++; yymore();
<OPTIONS>"::"           |
<OMESG>"::"             {
                            yytext[yyleng-2] = '\0';
                            BEGIN(ACTION);
                            return OPTION;
                        }
<OPTIONS>.              yymore(); BEGIN(OMESG);
<OMESG>.                yymore();

<ACTION>\n              yylinenum++;
<ACTION>[ \t]*

<ACTION>"end"           return END;
<ACTION>"goto"          return GOTO;
<ACTION>{LABEL}         return TAG;

<ACTION>;               {
                            numopt++;
                            BEGIN(OPTIONS);
                            return SEMICOLON;
                        }

<ACTION>"}"             {
                            fprintf(stderr, "Lexer Error:%d:", yylinenum);
                            yyerror("option has no semicolon terminated");
                            BEGIN(INITIAL);
                            return RB;
                        }

%%

#ifdef STANDALONE
int main(int argc, char *argv[])
{
    int i, y;
    for (i = 1; i < argc; i++) {
        yyfpath=argv[i];
        if (!(yyin = fopen(yyfpath, "r"))) {
            fprintf(stderr, "cannot open adventure book (%s)\n", argv[i]);
            exit(1);
        } 
        printf("open adventure book (%s)\n", argv[i]);
        while (1) {
            y = yylex();
            if (y >= TAG && y <= SEMICOLON) {
                printf("%s:%4d: (%d)\n", yyfpath, yylinenum, y);
            } else if (y == 0) {
                printf("Successfully done.\n");
                break;
            } else {
                fprintf(stderr, "cannot read adventure book (%s:%d) error=%d\n", argv[i], yylinenum, y);
                exit(1);
            }
        }
    }
}

void yyerror (const char *s) {
   fprintf (stderr, "%s (%s:%d)\n", s, yyfpath, yylinenum);
}
#endif
