#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef __APPLE__
#include <getopt.h>
#elif __linux__
#include <getopt.h>
#elif defined(_POSIX_VERSION)
#include <usistd.h>
#else
#include <getopt.h>
#endif

extern int yynerrs;
extern int yylinenum;
extern FILE *yyin;
extern int yyparse();

char *yyfpath;
void yyerror(const char *);
int set_debug_game(int);
extern int text_adventure(char *);

int main(int argc, char *argv[])
{
    int opt, d;
    char *storepos=NULL;
    while ((opt = getopt(argc, argv, "d:s:")) != -1) {
        switch(opt) {
          case 'd': // debug level
            d=atoi(optarg);
            set_debug_game(d);
            printf("debug level = %d\n", d);
            break;
          case 's': // store the position
            storepos = optarg;
            break;
          default: 
            fprintf(stderr, "\t%s -d <debug level> -s <store position tag> book [... book]\n", argv[0]);
            exit(1);
        }
    }

    int i;
    for (i = optind; i < argc; i++) {
        if (!(yyin = fopen(argv[i], "r"))) {
            fprintf(stderr, "cannot open adventure book (%s)\n", argv[i]);
            exit(1);
        }
        yyfpath=argv[i];
        if (yyparse() || yynerrs) {
            yyerror("main: parse error");
            printf("\n...? SD\n");
            exit(1);
        }
    }
    text_adventure(storepos);
    printf("\n---SD\n");
}

void yyerror (const char *s) {
   fprintf (stderr, "%s (%s:%d)\n", s, yyfpath, yylinenum);
}
